#include <math.h>
#include <stdio.h>
#include "mex.h"
#include "matrix.h"

// 1D Convolution operator
void conv_row(double *x,
          int lx,
          double *h,
          int lh,
          int ir,
          int M,
          double *y) {
    int i, j;
    double x0;
    for (i=lh-2; i > -1; i--)
        x[i] = x[lx+i];
    for (i=0; i<lx; i++){
        x0 = 0;
        for(j=0;j<lh;j++){
            x0 = x0 + x[j+i]*h[lh-1-j];
        }    
        y[M*i+ir] = x0; 
    }
}
void conv_col(double *x,
          int lx,
          double *h,
          int lh,
          int ic,
          int M,
          double *y) {
    int i, j;
    double x0;
    for (i=lh-2; i > -1; i--)
        x[i] = x[lx+i];
    for (i=0; i<lx; i++){
        x0 = 0;
        for(j=0;j<lh;j++){
            x0 = x0 + x[j+i]*h[lh-1-j];
        }    
        y[M*ic+i] = x0; 
    }
}

// Main Function
void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])

{
  double *x, *hr, *hc, *y;
  double *row, *col;
  int M, N, lh;
  int ir, ic;

  // Check for correct # of input variables
  if (nrhs!=3){
    mexErrMsgTxt("There must be 3 input parameters!");
    return;
  }
  x    = mxGetPr(prhs[0]);
  hr   = mxGetPr(prhs[1]);
  hc   = mxGetPr(prhs[2]);
  N    = mxGetN(prhs[0]); 
  M    = mxGetM(prhs[0]); 
  lh   = mxGetN(prhs[1]);
  plhs[0] = mxCreateDoubleMatrix(M,N,mxREAL);
  y = mxGetPr(plhs[0]);

  row = (double *)mxCalloc(N+lh-1,sizeof(double));
  col = (double *)mxCalloc(M+lh-1,sizeof(double));  
  if(N==1){     /* 1D signal */
      N = M;
      M = 1;
  }
  // Go by rows
  for(ir=0;ir<M;ir++){
      for(ic=0;ic<N;ic++){
          row[lh-1+ic] = x[M*ic+ir];
      }
      conv_row(row,N,hr,lh,ir,M,y);
  }
  if(M>1){
      // Go by columns
      for(ic=0;ic<N;ic++){
          for(ir=0;ir<M;ir++){
              col[lh-1+ir] = y[M*ic+ir];
          }
          conv_col(col, M, hc, lh, ic, M, y);
      }
  }
  mxFree(row);
  mxFree(col);
}